@@ -1,9 +1,9 @@
 /*@z04.c:Token Service:NewToken(), CopyTokenList()@***************************/
 /*                                                                           */
-/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.26)                       */
+/*  THE LOUT DOCUMENT FORMATTING SYSTEM (VERSION 3.27)                       */
 /*  COPYRIGHT (C) 1991, 2002 Jeffrey H. Kingston                             */
 /*                                                                           */
-/*  Jeffrey H. Kingston (jeff@cs.usyd.edu.au)                                */
+/*  Jeffrey H. Kingston (jeff@it.usyd.edu.au)                                */
 /*  Basser Department of Computer Science                                    */
 /*  The University of Sydney 2006                                            */
 /*  AUSTRALIA                                                                */
@@ -195,6 +195,7 @@ FULL_CHAR *EchoToken(OBJECT x)
     case BREAK:
     case UNDERLINE:
     case COLOUR:
+    case TEXTURE:
     case OUTLINE:
     case LANGUAGE:
     case CURR_LANG:
@@ -218,6 +219,7 @@ FULL_CHAR *EchoToken(OBJECT x)
     case GRAPHIC:
     case LINK_SOURCE:
     case LINK_DEST:
+    case LINK_DEST_NULL:
     case LINK_URL:
     case ACAT:
     case HCAT:
